@@ -251,8 +251,7 @@ def histo_cut(prec_accum):
             prec_accum_masked)].ravel(), bins=50)
         # get the class with biggest occurence
         index = np.where(n == n.max())
-        index = index[0]
-
+        index = index[0][0]
         # separated stop criterion check in case one of the bounds
         # is already robust
         if abs(lower_bound - lower_bound_before) > 1:
@@ -335,7 +334,9 @@ def classify_echo_fuzzy(dat, weights=None, trpz=None, thresh=0.5):
 
     """
     # Check the inputs
-    keys = ["zdr", "rho", "phi", "dop", "map"]
+    dkeys = ["zdr", "rho", "phi", "dop", "map"]
+    wkeys = ["zdr", "rho", "rho2", "phi", "dop", "map"]
+    tkeys = ["zdr", "rho", "rho2", "phi", "dop", "map"]
 
     if weights is None:
         weights = {"zdr": 0.4, "rho": 0.4, "rho2": 0.4,
@@ -349,17 +350,17 @@ def classify_echo_fuzzy(dat, weights=None, trpz=None, thresh=0.5):
                 "dop": [-0.2, -0.1, 0.1, 0.2],
                 "map": [1, 1, 9999, 9999]}
 
-    assert np.all(np.in1d(keys, dat.keys())), \
+    assert np.all(np.in1d(dkeys, list(dat.keys()))), \
         "Argument dat of classify_echo_fuzzy must be a dictionary " \
-        "with keywords %r." % (keys,)
-    assert np.all(np.in1d(keys, weights.keys())), \
+        "with keywords %r." % (dkeys,)
+    assert np.all(np.in1d(wkeys, list(weights.keys()))), \
         "Argument weights of classify_echo_fuzzy must be a dictionary " \
-        "with keywords %r." % (keys,)
-    assert np.all(np.in1d(keys, trpz.keys())), \
+        "with keywords %r." % (wkeys,)
+    assert np.all(np.in1d(tkeys, list(trpz.keys()))), \
         "Argument trpz of classify_echo_fuzzy must be a dictionary " \
-        "with keywords %r." % (keys,)
+        "with keywords %r." % (tkeys,)
     shape = None
-    for key in keys:
+    for key in dkeys:
         if not dat[key] is None:
             if shape is None:
                 shape = dat[key].shape
