@@ -24,6 +24,7 @@ for an introduction on how to deal with different file formats.
    read_RADOLAN_composite
    read_Rainbow
    read_safnwc
+   write_raster_dataset
    to_AAIGrid
    to_GeoTIFF
    to_hdf5
@@ -796,7 +797,7 @@ def read_radolan_header(fid):
     return header
 
 
-def read_RADOLAN_composite(fname, missing=-9999, loaddata=True):
+def read_RADOLAN_composite(f, missing=-9999, loaddata=True):
     """Read quantitative radar composite format of the German Weather Service
 
     The quantitative composite format of the DWD (German Weather Service) was
@@ -820,8 +821,8 @@ def read_RADOLAN_composite(fname, missing=-9999, loaddata=True):
 
     Parameters
     ----------
-    fname : string
-        path to the composite file
+    f : string or file handle
+        path to the composite file or file handle
     missing : int
         value assigned to no-data cells
     loaddata : bool
@@ -846,9 +847,12 @@ def read_RADOLAN_composite(fname, missing=-9999, loaddata=True):
     NODATA = missing
     mask = 0xFFF  # max value integer
 
-    f = get_radolan_filehandle(fname)
-
-    header = read_radolan_header(f)
+    # If a file name is supplied, get a file handle
+    try:
+        header = read_radolan_header(f)
+    except AttributeError:
+        f = get_radolan_filehandle(f)
+        header = read_radolan_header(f)
 
     attrs = parse_DWD_quant_composite_header(header)
 
@@ -1058,7 +1062,7 @@ def read_gamic_scan_attributes(scan, scan_type):
         ele_start = np.round(ray_header['elevation_start'], 1)
         ele_stop = np.round(ray_header['elevation_stop'], 1)
         angle_step = np.round(sattrs['angle_step'], 1)
-        angle_step = np.round(sattrs['ele_stop'], 1) / angle_step
+        angle_step = int(np.round(sattrs['ele_stop'], 1) / angle_step)
         # Elevation corresponding to 1st ray
         if ele_start[0] < 0:
             ele_start = ele_start[1:]
@@ -1129,7 +1133,7 @@ def read_gamic_scan(scan, scan_type, wanted_moments):
                 mdata = sg2[...]
                 dyn_range_max = sg2.attrs.get('dyn_range_max')
                 dyn_range_min = sg2.attrs.get('dyn_range_min')
-                bin_format = sg2.attrs.get('format')
+                bin_format = sg2.attrs.get('format').decode()
                 if bin_format == 'UV8':
                     div = 256.0
                 else:
@@ -1538,15 +1542,15 @@ def get_RB_blob_from_string(datastring, blobdict):
     return data
 
 
-def get_RB_blob_from_file(filename, blobdict):
+def get_RB_blob_from_file(f, blobdict):
     """
     Read BLOB data from file and return it with correct
     dataWidth and shape
 
     Parameters
     ----------
-    filename : string
-        Filename of Data File
+    f : string or file handle
+        File handle of or path to Rainbow file
     blobdict : dict
         Blob Dict
 
@@ -1555,53 +1559,58 @@ def get_RB_blob_from_file(filename, blobdict):
     data : numpy array
         Content of blob as numpy array
     """
-    try:
-        fid = open(filename, "rb")
-    except IOError:
-        print("WRADLIB: Error opening Rainbow file ", filename)
-        raise IOError
 
-    datastring = fid.read()
-    fid.close()
+    # Try to read the data from a file handle
+    try:
+        f.seek(0, 0)
+        fid = f
+        datastring = fid.read()
+    except AttributeError:
+        # If we did not get a file handle, assume that we got a filename,
+        # get a file handle and read the data
+        try:
+            fid = open(f, "rb")
+            datastring = fid.read()
+            fid.close()
+        except IOError:
+            print("WRADLIB: Error opening Rainbow file ", f)
+            raise IOError
 
     data = get_RB_blob_from_string(datastring, blobdict)
 
     return data
 
 
-def get_RB_file_as_string(filename):
+def get_RB_file_as_string(fid):
     """ Read Rainbow File Contents in dataString
 
     Parameters
     ----------
-    filename : string
-        Filename of Data File
+    fid : file handle
+        File handle of Data File
 
     Returns
     -------
     dataString : string
         File Contents as dataString
     """
-    try:
-        fid = open(filename, "rb")
-    except IOError:
-        print("WRADLIB: Error opening Rainbow file ", filename)
-        raise IOError
 
-    dataString = fid.read()
-    fid.close()
+    try:
+        dataString = fid.read()
+    except:
+        raise IOError('Could not read from file handle')
 
     return dataString
 
 
-def get_RB_blobs_from_file(filename, rbdict):
+def get_RB_blobs_from_file(fid, rbdict):
     """Read all BLOBS found in given nested dict, loads them from file
     given by filename and add them to the dict at the appropriate position.
 
     Parameters
     ----------
-    filename : string
-        Filename of Data File
+    fid : file handle
+        File handle of Data File
     rbdict : dict
         Rainbow file Contents
 
@@ -1613,7 +1622,7 @@ def get_RB_blobs_from_file(filename, rbdict):
 
     blobs = list(find_key('@blobid', rbdict))
 
-    datastring = get_RB_file_as_string(filename)
+    datastring = get_RB_file_as_string(fid)
     for blob in blobs:
         data = get_RB_blob_from_string(datastring, blob)
         blob['data'] = data
@@ -1621,13 +1630,13 @@ def get_RB_blobs_from_file(filename, rbdict):
     return rbdict
 
 
-def get_RB_header(filename):
+def get_RB_header(fid):
     """Read Rainbow Header from filename, converts it to a dict and returns it
 
     Parameters
     ----------
-    filename : string
-        Filename of Data File
+    fid : file handle
+        File handle of Data File
 
     Returns
     -------
@@ -1635,31 +1644,28 @@ def get_RB_header(filename):
         Rainbow File Contents
 
     """
-    try:
-        fid = open(filename, "rb")
-    except IOError:
-        print("WRADLIB: Error opening Rainbow file ", filename)
-        raise IOError
 
     # load the header lines, i.e. the XML part
     endXMLmarker = b"<!-- END XML -->"
     header = b""
     line = b""
-    while not line.startswith(endXMLmarker):
-        header += line[:-1]
-        line = fid.readline()
-        if len(line) == 0:
-            break
 
-    fid.close()
+    try:
+        while not line.startswith(endXMLmarker):
+            header += line[:-1]
+            line = fid.readline()
+            if len(line) == 0:
+                break
+    except:
+        raise IOError('Could not read from file handle')
 
     xmltodict = util.import_optional('xmltodict')
 
     return xmltodict.parse(header)
 
 
-def read_Rainbow(filename, loaddata=True):
-    """"Reads Rainbow files files according to their structure
+def read_Rainbow(f, loaddata=True):
+    """Reads Rainbow files files according to their structure
 
     In contrast to other file readers under :meth:`wradlib.io`, this function
     will *not* return a two item tuple with (data, metadata). Instead, this
@@ -1673,8 +1679,8 @@ def read_Rainbow(filename, loaddata=True):
 
     Parameters
     ----------
-    filename : string
-        a rainbow file path
+    f : string or file handle
+        a rainbow file path or file handle of rainbow file
     loaddata : bool
         True | False, If False function returns only metadata
 
@@ -1689,12 +1695,28 @@ def read_Rainbow(filename, loaddata=True):
 
     See :ref:`notebooks/fileio/wradlib_load_rainbow_example.ipynb`.
 
+    .. versionchanged 0.10.0
+       Added reading from file handles.
+
     """
 
-    rbdict = get_RB_header(filename)
+    # Check if a file handle has been passed
+    try:
+        f.seek(0, 0)
+        fid = f
+    except AttributeError:
+        # If we did not get a file handle, assume that we got a filename and
+        #  get a file handle for the corresponding file
+        try:
+            fid = open(f, "rb")
+        except IOError:
+            print("WRADLIB: Error opening Rainbow file ", f)
+            raise IOError
+
+    rbdict = get_RB_header(fid)
 
     if loaddata:
-        rbdict = get_RB_blobs_from_file(filename, rbdict)
+        rbdict = get_RB_blobs_from_file(fid, rbdict)
 
     return rbdict
 
@@ -1795,17 +1817,101 @@ def read_safnwc(filename):
 
     # name = os.path.basename(filename)[7:11]
     try:
-        proj = root.GetMetadata()["PROJECTION"]
+        proj = osr.SpatialReference()
+        proj.ImportFromProj4(ds.GetMetadata()["PROJECTION"])
     except Exception:
         raise NameError("No metadata for satellite file %s" % filename)
     geotransform = root.GetMetadata()["GEOTRANSFORM_GDAL_TABLE"].split(",")
     geotransform[0] = root.GetMetadata()["XGEO_UP_LEFT"]
     geotransform[3] = root.GetMetadata()["YGEO_UP_LEFT"]
-    ds.SetProjection(proj)
+    ds.SetProjection(proj.ExportToWkt())
     ds.SetGeoTransform([float(x) for x in geotransform])
     return ds
 
 
+def read_netcdf_group(ncid):
+    """Reads netcdf (nested) groups into python dictionary with corresponding
+    structure.
+
+    Note
+    ----
+    The returned dictionary could be quite big, depending on the content of
+    the file.
+
+    Parameters
+    ----------
+    ncid : object
+        nc/group id from netcdf file
+
+    Returns
+    -------
+    out : ordered dict
+        an ordered dictionary that contains both data and metadata
+        according to the original netcdf file structure
+    """
+    out = OrderedDict()
+
+    # attributes
+    for k, v in ncid.__dict__.items():
+        out[k] = v
+
+    # groups
+    if ncid.groups:
+        for k, v in ncid.groups.items():
+            out[k] = read_netcdf_group(v)
+
+    # dimensions
+    dimids = np.array([])
+    if ncid.dimensions:
+        dim = OrderedDict()
+        for k, v in ncid.dimensions.items():
+            tmp = OrderedDict()
+            try:
+                tmp['data_model'] = v._data_model
+            except AttributeError:
+                pass
+            try:
+                tmp['size'] = v.__len__()
+            except AttributeError:
+                pass
+            tmp['dimid'] = v._dimid
+            dimids = np.append(dimids, v._dimid)
+            tmp['grpid'] = v._grpid
+            tmp['isunlimited'] = v.isunlimited()
+            dim[k] = tmp
+        # Usually, the dimensions should be ordered by dimid automatically
+        # in case netcdf used OrderedDict. However, we should double check
+        if np.array_equal(dimids, np.sort(dimids)):
+            # is already sorted
+            out['dimensions'] = dim
+        else:
+            # need to sort
+            dim2 = OrderedDict()
+            keys = dim.keys()
+            for dimid in np.sort(dimids):
+                dim2[keys[dimid]] = dim[keys[dimid]]
+            out["dimensions"] = dim2
+
+    # variables
+    if ncid.variables:
+        var = OrderedDict()
+        for k, v in ncid.variables.items():
+            tmp = OrderedDict()
+            for k1 in v.ncattrs():
+                tmp[k1] = v.getncattr(k1)
+            if v[:].dtype.kind == 'S':
+                try:
+                    tmp['data'] = nc.chartostring(v[:])
+                except:
+                    tmp['data'] = v[:]
+            else:
+                tmp['data'] = v[:]
+            var[k] = tmp
+        out['variables'] = var
+
+    return out
+
+
 def read_generic_netcdf(fname):
     """Reads netcdf files and returns a dictionary with corresponding
     structure.
@@ -1852,68 +1958,7 @@ def read_generic_netcdf(fname):
         print("Raising exception...")
         raise
 
-    if ncid.groups:
-        # To be implemented if necessary, all netcdf files
-        # I got my hands on have just one group/Dataset
-        print("Groups", ncid.groups)
-
-    out = OrderedDict()
-
-    # get file format (should be NETCDF3 or NETCDF4)
-    try:
-        out["file_format"] = ncid.file_format
-    except AttributeError:
-        pass
-
-    # global attributes
-    for k, v in ncid.__dict__.items():
-        out[k] = v
-
-    # dimensions
-    dimids = np.array([])
-    if ncid.dimensions:
-        dim = OrderedDict()
-        for k, v in ncid.dimensions.items():
-            tmp = OrderedDict()
-            try:
-                tmp['data_model'] = v._data_model
-            except AttributeError:
-                pass
-            try:
-                tmp['size'] = v.__len__()
-            except AttributeError:
-                pass
-            tmp['dimid'] = v._dimid
-            dimids = np.append(dimids, v._dimid)
-            tmp['grpid'] = v._grpid
-            tmp['isunlimited'] = v.isunlimited()
-            dim[k] = tmp
-        # Usually, the dimensions should be ordered by dimid automatically
-        # in case netcdf used OrderedDict. However, we should double check
-        if np.array_equal(dimids, np.sort(dimids)):
-            # is already sorted
-            out['dimensions'] = dim
-        else:
-            # need to sort
-            dim2 = OrderedDict()
-            keys = dim.keys()
-            for dimid in np.sort(dimids):
-                dim2[keys[dimid]] = dim[keys[dimid]]
-            out["dimensions"] = dim2
-
-    # variables
-    if ncid.variables:
-        var = OrderedDict()
-        for k, v in ncid.variables.items():
-            tmp = OrderedDict()
-            for k1 in v.ncattrs():
-                tmp[k1] = v.getncattr(k1)
-            if v[:].dtype.kind == 'S':
-                tmp['data'] = nc.chartostring(v[:])
-            else:
-                tmp['data'] = v[:]
-            var[k] = tmp
-        out['variables'] = var
+    out = read_netcdf_group(ncid)
 
     ncid.close()
 
@@ -1937,6 +1982,54 @@ def _check_arguments(fpath, data):
                         os.path.dirname(fpath))
 
 
+def write_raster_dataset(fpath, dataset, format, options=None, remove=False):
+    """ Write raster dataset to file format
+
+    .. versionadded 0.10.0
+
+    Parameters
+    ----------
+    fpath : string
+        A file path - should have file extension corresponding to format.
+    dataset : gdal.Dataset
+        gdal raster dataset
+    format : string
+        gdal raster format string
+    options : list
+        List of option strings for the corresponding format.
+    remove : bool
+        if True, existing OGR.DataSource will be
+        removed before creation
+
+    Note
+    ----
+    For format and options refer to
+    `formats_list <http://www.gdal.org/formats_list.html>`_.
+
+    Examples
+    --------
+    See :ref:`notebooks/fileio/wradlib_gis_export_example.ipynb`.
+    """
+    # check for option list
+    if options is None:
+        options = []
+
+    driver = gdal.GetDriverByName(format)
+    metadata = driver.GetMetadata()
+
+    # check driver capability
+    if 'DCAP_CREATECOPY' in metadata and metadata['DCAP_CREATECOPY'] != 'YES':
+        assert "Driver %s doesn't support CreateCopy() method.".format(format)
+
+    if remove:
+        if os.path.exists(fpath):
+            driver.Delete(fpath)
+
+    target = driver.CreateCopy(fpath, dataset, 0, options)
+    del target
+
+
+@util.deprecated(write_raster_dataset)
 def to_AAIGrid(fpath, data, xllcorner, yllcorner, cellsize,
                nodata=-9999, proj=None, fmt="%.2f", to_esri=True):
     """Write a cartesian grid to an Arc/Info ASCII grid file.
@@ -2042,6 +2135,7 @@ def to_AAIGrid(fpath, data, xllcorner, yllcorner, cellsize,
     return 0
 
 
+@util.deprecated(write_raster_dataset)
 def to_GeoTIFF(fpath, data, geotransform, nodata=-9999, proj=None):
     """Write a cartesian grid to a GeoTIFF file.
 
@@ -2170,6 +2264,7 @@ def to_GeoTIFF(fpath, data, geotransform, nodata=-9999, proj=None):
     ds = None
 
 
+@util.deprecated(georef.extract_raster_dataset)
 def read_raster_data(filename, driver=None, **kwargs):
     """Read raster data
 
